=== modified file 'drizzled/message/statement_transform.cc'
--- drizzled/message/statement_transform.cc	2010-07-29 15:50:13 +0000
+++ drizzled/message/statement_transform.cc	2010-07-29 15:50:18 +0000
@@ -1061,18 +1061,11 @@
     {
       if (part.has_compare_length())
       {
-        size_t compare_length_in_chars= part.compare_length();
-        
-        /* hack: compare_length() is bytes, not chars, but
-         * only for VARCHAR. Ass. */
-        if (field.type() == Table::Field::VARCHAR)
-          compare_length_in_chars/= 4;
-
-        if (compare_length_in_chars != field.string_options().length())
+        if (part.compare_length() != field.string_options().length())
         {
           stringstream ss;
           destination.push_back('(');
-          ss << compare_length_in_chars;
+          ss << part.compare_length();
           destination.append(ss.str());
           destination.push_back(')');
         }

=== modified file 'drizzled/table_proto_write.cc'
--- drizzled/table_proto_write.cc	2010-07-30 19:21:23 +0000
+++ drizzled/table_proto_write.cc	2010-07-30 19:22:02 +0000
@@ -35,6 +35,7 @@
 #include <google/protobuf/message.h>
 
 #include <drizzled/table_proto.h>
+#include <drizzled/charset.h>
 
 #include "drizzled/function/time/typecast.h"
 
@@ -454,12 +455,29 @@
     for(unsigned int j=0; j< key_info[i].key_parts; j++)
     {
       message::Table::Index::IndexPart *idxpart;
+      const int fieldnr= key_info[i].key_part[j].fieldnr;
+      int mbmaxlen= 1;
 
       idxpart= idx->add_index_part();
 
-      idxpart->set_fieldnr(key_info[i].key_part[j].fieldnr);
-
-      idxpart->set_compare_length(key_info[i].key_part[j].length);
+      idxpart->set_fieldnr(fieldnr);
+
+      if (table_proto.field(fieldnr).type() == message::Table::Field::VARCHAR
+          || table_proto.field(fieldnr).type() == message::Table::Field::BLOB)
+      {
+        uint32_t collation_id;
+
+        if (table_proto.field(fieldnr).string_options().has_collation_id())
+          collation_id= table_proto.field(fieldnr).string_options().collation_id();
+        else
+          collation_id= table_proto.options().collation_id();
+
+        const CHARSET_INFO *cs= get_charset(collation_id);
+
+        mbmaxlen= cs->mbmaxlen;
+      }
+
+      idxpart->set_compare_length(key_info[i].key_part[j].length / mbmaxlen);
     }
   }
 

=== modified file 'drizzled/table_share.cc'
--- drizzled/table_share.cc	2010-08-04 23:31:14 +0000
+++ drizzled/table_share.cc	2010-08-05 19:24:12 +0000
@@ -999,6 +999,24 @@
 
       key_part->length= part.compare_length();
 
+      int mbmaxlen= 1;
+
+      if (table.field(part.fieldnr()).type() == message::Table::Field::VARCHAR
+          || table.field(part.fieldnr()).type() == message::Table::Field::BLOB)
+      {
+        uint32_t collation_id;
+
+        if (table.field(part.fieldnr()).string_options().has_collation_id())
+          collation_id= table.field(part.fieldnr()).string_options().collation_id();
+        else
+          collation_id= table.options().collation_id();
+
+        const CHARSET_INFO *cs= get_charset(collation_id);
+
+        mbmaxlen= cs->mbmaxlen;
+      }
+      key_part->length*= mbmaxlen;
+
       key_part->store_length= key_part->length;
 
       /* key_part->offset is set later */

